DROP DATABASE dmdb;
CREATE DATABASE dmdb DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;
USE dmdb;

-- tabulka pouzitych jazyku
CREATE TABLE languages (
  code VARCHAR(3) NOT NULL PRIMARY KEY,
    -- kod jazyku (cz, en, de, it, atd...)
  name VARCHAR(255) NOT NULL);
    -- nazev jazyku bude vsude standardne v anglictine

-- tabulka lokalit
CREATE TABLE locations (
  id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
  parent INT NULL, FOREIGN KEY (parent) REFERENCES locations(id) ON DELETE RESTRICT,
    -- rodicovska lokalita. Pokud je null, jedna se o kontinent, dalsi uroven jsou staty a pak zeme
  lang VARCHAR(3) NULL, FOREIGN KEY (lang) REFERENCES languages(code) ON DELETE RESTRICT);
    -- jedna li se o stat, uvadi se jeste statni jazyk, jinak null

-- tabulka nazvu lokalit pro jednotlive jazyky
CREATE TABLE locnames (
  location INT NOT NULL, FOREIGN KEY (location) REFERENCES locations(id) ON DELETE RESTRICT,
  lang VARCHAR(3) NOT NULL, FOREIGN KEY (lang) REFERENCES languages(code) ON DELETE RESTRICT,
  PRIMARY KEY (location, lang),
  name VARCHAR(255) NOT NULL);

-- tabulka kategorii
CREATE TABLE categories (
  id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
  parent INT NULL, FOREIGN KEY (parent) REFERENCES categories(id) ON DELETE RESTRICT);
    -- rodicovska kategorie. Pokud je null, tak se jedna o korenovou kategorii

-- tabulka nazvu kategorii pro jednotlive jazyky
CREATE TABLE catnames (
  category INT NOT NULL, FOREIGN KEY (category) REFERENCES categories(id) ON DELETE RESTRICT,
  lang VARCHAR(3) NOT NULL, FOREIGN KEY (lang) REFERENCES languages(code) ON DELETE RESTRICT,
  PRIMARY KEY (category, lang),
  name VARCHAR(255) NOT NULL);

-- tabulka uzivatel (administratori a dodavatele)
CREATE TABLE users (
  id BIGINT NOT NULL PRIMARY KEY AUTO_INCREMENT,
  name VARCHAR(255) NOT NULL,
    -- jmeno firmy nebo jmeno a prijmeni odberatele
  address1 VARCHAR(255) NOT NULL,
    -- adresa pole 1
  address2 VARCHAR(255) NOT NULL,
    -- adresa pole 2
  city VARCHAR(255) NOT NULL,
    -- mesto
  postalcode VARCHAR(20) NOT NULL,
    -- psc
  state VARCHAR(255) NOT NULL,
    -- stat
  person BOOL NOT NULL,
    -- je fyzicka osoba
  ico VARCHAR(50) NULL,
    -- jen pro firmy
  dic VARCHAR(50) NULL,
    -- jen pro firmy
  email VARCHAR(50) NOT NULL, UNIQUE (email),
    -- email
  password VARCHAR(50) NOT NULL,
    -- heslo
  passwordtime TIMESTAMP NULL,
    -- doba vygenerovani hesla. heslo se musi zmenit do nejake doby
    -- od jeho vygenerovani. pokud je nastaveno na null, nemusi se uz menit
  telephone VARCHAR(20) NULL,
    -- nepovinny telefon
  role INT NOT NULL);
    -- 0 = admin, 1 = odberatel, 2 = dodavatel

-- tabulka poptavek
CREATE TABLE demands (
  id BIGINT NOT NULL PRIMARY KEY AUTO_INCREMENT,
  price INT NOT NULL,
    -- odhadovana cena poptavky
      -- 1 - do 100 000
      -- 2 - 100 000 - 500 000
      -- 3 - 500 000 - 5 000 000
      -- 4 - nad 5 000 000
  currency INT NOT NULL,
    -- mena, ve ktere je cena uvedena, 0 - dolary, 1 - eura
  creation TIMESTAMP NOT NULL, INDEX (creation),
    -- datum vytvoreni
  expiration DATE NULL,
    -- platnost (null pokud je neomezena)
  confirmed BOOL NOT NULL, INDEX (confirmed),
    -- priznak urcuje, zda li je poptavka potvrzena a zobrazuje se pro dodavatele
  valid BOOL NOT NULL, INDEX (valid),
    -- pokud je true a vyprsel cas platnosti, nastavi se na false a odesle se email
  filename1 VARCHAR(255) NULL,
    -- nazev prilohy 1
  filename2 VARCHAR(255) NULL,
    -- nazev prilohy 2
  filename3 VARCHAR(255) NULL,
    -- nazev prilohy 3
  lang VARCHAR(3) NOT NULL, FOREIGN KEY (lang) REFERENCES languages(code) ON DELETE RESTRICT,
    -- puvodni jazyk zadani poptavky, ktery se zachova v pripade zmen a ostatni se smazi
  owner BIGINT NOT NULL, FOREIGN KEY (owner) REFERENCES users(id) ON DELETE RESTRICT);
    -- zadavatel poptavky

-- tabulka popisu poptavek pro jednotlive jazyky
CREATE TABLE demanddescriptions (
  demand BIGINT NOT NULL, FOREIGN KEY (demand) REFERENCES demands(id) ON DELETE RESTRICT,
  lang VARCHAR(3) NOT NULL, FOREIGN KEY (lang) REFERENCES languages(code) ON DELETE RESTRICT,
  PRIMARY KEY (demand, lang),
  name VARCHAR(255) NOT NULL,
  text TEXT NOT NULL);

-- spojovaci tabulka pro poptavky a lokality, ktere pokryva
CREATE TABLE demandlocations (
  demand BIGINT NOT NULL, FOREIGN KEY (demand) REFERENCES demands(id) ON DELETE RESTRICT,
  location INT NOT NULL, FOREIGN KEY (location) REFERENCES locations(id) ON DELETE RESTRICT,
  PRIMARY KEY (demand, location));

-- spojovaci tabulka pro poptavky a kategorie, do kterych spada
CREATE TABLE demandcategories (
  demand BIGINT NOT NULL, FOREIGN KEY (demand) REFERENCES demands(id) ON DELETE RESTRICT,
  category INT NOT NULL, FOREIGN KEY (category) REFERENCES categories(id) ON DELETE RESTRICT,
  PRIMARY KEY (demand, category));

-- spojovaci tabulka pro poptavky a lokality, ktere pokryva
CREATE TABLE userlocations (
  user BIGINT NOT NULL, FOREIGN KEY (user) REFERENCES users(id) ON DELETE RESTRICT,
  location INT NOT NULL, FOREIGN KEY (location) REFERENCES locations(id) ON DELETE RESTRICT,
  PRIMARY KEY (user, location));

-- spojovaci tabulka pro poptavky a kategorie, do kterych spada
CREATE TABLE usercategories (
  user BIGINT NOT NULL, FOREIGN KEY (user) REFERENCES users(id) ON DELETE RESTRICT,
  category INT NOT NULL, FOREIGN KEY (category) REFERENCES categories(id) ON DELETE RESTRICT,
  PRIMARY KEY (user, category));
